/*
===========================================================================
Copyright (C) 1999-2005 Id Software, Inc.

This file is part of Quake III Arena source code.

Quake III Arena source code is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the License,
or (at your option) any later version.

Quake III Arena source code is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Quake III Arena source code; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
===========================================================================
*/

#define CONTEXT_ALL						0xFFFFFFFF
#define CONTEXT_NORMAL					1
#define CONTEXT_NEARBYITEM				2
#define CONTEXT_CTFREDTEAM				4
#define CONTEXT_CTFBLUETEAM				8
#define CONTEXT_REPLY					16
#define CONTEXT_OBELISKREDTEAM			32
#define CONTEXT_OBELISKBLUETEAM			64
#define CONTEXT_HARVESTERREDTEAM		128
#define CONTEXT_HARVESTERBLUETEAM		256

#define CONTEXT_NAMES 1024
